=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-08-21 18:55:40 +0000
+++ bzrlib/errors.py	2006-08-21 19:05:37 +0000
@@ -126,7 +126,12 @@
 
     def __str__(self):
         try:
-            return self.__doc__ % self.__dict__
+            # __str__() should always return a 'str' object
+            # never a 'unicode' object.
+            s = self.__doc__ % self.__dict__
+            if isinstance(s, unicode):
+                return s.encode('utf8')
+            return s
         except (NameError, ValueError, KeyError), e:
             return 'Unprintable exception %s: %s' \
                 % (self.__class__.__name__, str(e))

=== modified file 'bzrlib/tests/test_errors.py'
--- bzrlib/tests/test_errors.py	2006-06-15 05:36:34 +0000
+++ bzrlib/tests/test_errors.py	2006-08-21 19:05:37 +0000
@@ -17,9 +17,11 @@
 
 """Tests for the formatting and construction of errors."""
 
-import bzrlib.bzrdir as bzrdir
-import bzrlib.errors as errors
-from bzrlib.tests import TestCaseWithTransport
+from bzrlib import (
+    bzrdir,
+    errors,
+    )
+from bzrlib.tests import TestCase, TestCaseWithTransport
 
 
 class TestErrors(TestCaseWithTransport):
@@ -44,3 +46,23 @@
                              "Please run bzr reconcile on this repository." %
                              repo.bzrdir.root_transport.base,
                              str(error))
+
+
+class PassThroughError(errors.BzrNewError):
+    """Pass through %(foo)s and %(bar)s"""
+
+    def __init__(self, foo, bar):
+        errors.BzrNewError.__init__(self, foo=foo, bar=bar)
+
+
+class TestErrorFormatting(TestCase):
+    
+    def test_always_str(self):
+        e = PassThroughError(u'\xb5', 'bar')
+        self.assertIsInstance(e.__str__(), str)
+        # In Python str(foo) *must* return a real byte string
+        # not a Unicode string. The following line would raise a
+        # Unicode error, because it tries to call str() on the string
+        # returned from e.__str__(), and it has non ascii characters
+        s = str(e)
+        self.assertEqual('Pass through \xc2\xb5 and bar', s)

